/* eslint-disable @typescript-eslint/no-unsafe-member-access */
import { Message, TextChannel } from 'discord.js';
import messages from '../messages.json';
import { transferPin } from '../utils/common';

const save = async (
  message: Message, mainTextChannel: TextChannel, pinTextChannel: TextChannel, silent = true,
): Promise<void> => {
  const authorMember = await message.guild?.members.fetch(message.author);
  if (!authorMember) {
    await message.reply(messages.INVALID_PERMISSIONS);
    return;
  }

  const authorCanPin = authorMember.permissionsIn(mainTextChannel).has('MANAGE_MESSAGES');
  if (!authorCanPin) {
    await message.reply(messages.INVALID_PERMISSIONS);
    return;
  }

  const pinnedMessages = await mainTextChannel.messages.fetchPinned();
  const pinnedMessagesCollection = Array.from(pinnedMessages.values());
  if (pinnedMessagesCollection.length <= 0) {
    await message.reply(messages.NOTHING_TO_SAVE);
    return;
  }

  const promises = pinnedMessagesCollection
    .reverse()
    .map((reply, i) => new Promise<void>((resolve, reject) => {
      setTimeout(() => {
        transferPin(reply, pinTextChannel, silent)
          .then(resolve)
          .catch(reject);
      }, i * Number(process.env.DISCORD_MESSAGES_THROTTLE || '1000'));
    }));
  await Promise.all(promises);
  await message.reply(messages.SAVE_COMPLETE);
};

export default save;
