/* eslint-disable @typescript-eslint/no-unsafe-member-access, no-void */
import dotenv from 'dotenv';
import { Client, Intents, TextChannel } from 'discord.js';
import messages from './messages.json';
import { transferPin, getCommandPattern, log } from './utils/common';
import pin from './commands/pin';
import save from './commands/save';

dotenv.config();

const listen = async (channelIDMap: Map<string, string>) => {
  const client = new Client({ intents: [Intents.FLAGS.GUILDS, Intents.FLAGS.GUILD_MESSAGES] });
  client.on('ready', () => {
    log(`Logged in as ${client.user?.tag || ''}!`);
  });

  Array.from(channelIDMap).forEach(([mainChannelID, pinChannelID]) => {
    client.on('messageCreate', async (message) => {
      if (message.channel.id !== mainChannelID) {
        return;
      }

      const mainChannel = await client.channels.fetch(mainChannelID);
      const pinChannel = await client.channels.fetch(pinChannelID);
      if (mainChannel?.type !== 'GUILD_TEXT' || pinChannel?.type !== 'GUILD_TEXT') {
        return;
      }

      const mainTextChannel = mainChannel as TextChannel;
      const pinTextChannel = pinChannel as TextChannel;

      const clientUser = client.user;
      if (!clientUser) {
        return;
      }

      if (message.type === 'CHANNEL_PINNED_MESSAGE') {
        const reply = await message.fetchReference();
        await transferPin(reply, pinTextChannel, false);
        await reply.unpin();
        await mainTextChannel.send(`${message.author.toString()} ${messages.PIN_COMPLETE}`);
        await message.delete();
        return;
      }

      if (message.author.id === clientUser.id) {
        return;
      }

      const mentioned = message.mentions.users.has(clientUser.id);
      if (!mentioned) {
        return;
      }

      switch (message.content) {
        case getCommandPattern(clientUser, 'pin'):
          await pin(message, mainTextChannel, pinTextChannel);
          return;
        case getCommandPattern(clientUser, 'save'):
          await save(message, mainTextChannel, pinTextChannel);
          return;
        default:
          break;
      }

      await message.reply(messages.UNKNOWN_COMMAND);
    });
  });

  await client.login(process.env.DISCORD_BOT_TOKEN as string);
  return client;
};

const main = async (channelMap: Map<string, string>) => {
  await listen(channelMap);
};

const channelMap = new Map<string, string>();
channelMap.set(
  process.env.DISCORD_MAIN_CHANNEL as string,
  process.env.DISCORD_PIN_CHANNEL as string,
);
void main(channelMap);
